(function() {

    if (navigator.isCocoonJS) return;

    Timer = function() {
        this.reset();
        return this;
    };

    Timer.prototype = {
        currentTimeInMillis : 0,
        lastTimeInMillis : 0,
        elapsedTimeInMillis : 0,
        elapsedTimeInSeconds : 0,
        accumTimeInMillis : 0,

        reset : function() {
            this.currentTimeInMillis = this.lastTimeInMillis = new Date().getTime();
            this.accumTimeInMillis = this.elapsedTimeInMillis = this.elapsedTimeInSeconds = 0;
        },

        update : function() {
            this.currentTimeInMillis = new Date().getTime();
            this.elapsedTimeInMillis = this.currentTimeInMillis - this.lastTimeInMillis;
            this.elapsedTimeInSeconds = this.elapsedTimeInMillis / 1000.0;
            this.lastTimeInMillis = this.currentTimeInMillis;
            this.accumTimeInMillis += this.elapsedTimeInMillis;
        }
    };

    FPSCounter = function() {
        this.timer = new Timer();
        this.reset();
        return this;
    };

    FPSCounter.prototype = {
        timer : 0,
        fps : 0,
        averageFPS : 0,

        reset : function()
        {
            this.timer.reset();
            this.fps = 0;
            this.averageFPS = 0;
        },

        update : function()
        {
            this.timer.update();
            this.fps++;
            if (this.timer.accumTimeInMillis >= 1000) {
                this.averageFPS = ((1000 * this.fps / this.timer.accumTimeInMillis * 100) | 0) / 100;
                this.fps = 0;
                this.timer.reset();
            }
        }
    };

    window.addEventListener("load", function() {
        textColor = "white";
        fpsCounter = new FPSCounter();
        fpsDivParentDiv = document.createElement("div");
        var topOrBottom = true;
        var leftOrRight = false;
        fpsDivParentDiv.style.cssText = "position:absolute; " + (topOrBottom ? "top" : "bottom") + ":0%; width:100%; height:10%;";
        fpsDivParentDiv.innerHTML = "<div id='cocoonjs_fpsDiv' style='background:black; float:" + (leftOrRight ? "left" : "right") + "; color:" + textColor + "''></div>";
        document.body.appendChild(fpsDivParentDiv);
        fpsDiv = document.getElementById("cocoonjs_fpsDiv");
        fpsIntervalId = setInterval(function()
        {
            fpsCounter.update();
            fpsDiv.innerHTML = "< WV FPS: " + (fpsCounter.averageFPS) + "  >";
        }, 1000 / 60);
        // If we are inside CocoonJS webview, pause the CocoonJS loop
        if (typeof(window.ext) !== "undefined" && typeof(window.ext.IDTK_APP) !== "undefined") {
            window.ext.IDTK_APP.makeCallAsync("forward", "window.ext.IDTK_APP.makeCall('pause');");
        }
    });
})();
