(function() {

    CAAT.Foundation.ActorContainer.extend(
        {
            /**
             * @type : {CAAT.Foundation.Actor}
             */
            spinner : null,

            /**
             * @type : {boolean}
             */
            showing : false,

            /**
             * setTimeout identifier.
             * @type : {number}
             */
            layerShowId : null,

            /**
             * When the loading layer was shown.
             * @type {number}
             */
            showTime : null,

            /**
             * timer to show a cancel loading button.
             * @type CAAT.Foundation.Timer.TimerTask
             */
            buttonTTask : null,

            font : null,

            __init : function( director, spinnerImg, font ) {
                this.__super();

                var me= this;

                this.font= font;

                this.setSize( director.width, director.height ).
                    enableEvents(true).
                    setFillStyle("rgba(0,0,0,.7)").
                    setVisible(false).
                    setPreventLayout(true);

                var spinner = new CAAT.Foundation.Actor().
                    setPositionAnchor(.5, .5).
                    setFillStyle('#00f').
                    setSize(100, 100).
                    rescaleToScreen().
                    setLocation(this.width / 2, this.height / 2).
                    addBehavior(
                    new CAAT.Behavior.RotateBehavior().
                        setDelayTime(0, 1000).
                        setValues(0, 2 * Math.PI).
                        setCycle(true)
                );
                if (spinnerImg) {
                    spinner.setBackgroundImage(spinnerImg).rescaleToScreen();
                }

                this.buttonCancel= iBasket.Util.createButtonWithTextAndIcon(
                    new CAAT.Foundation.SpriteImage().initialize( director.getImage("score_botonpequeno"), 3, 1),
                    "Cancel",
                    director.getImage("menu_icon_play"),
                    this.font,
                    function() {
                        me.cancel();
                    });

                this.spinner= spinner;
                this.addChild(spinner);
                this.addChild(this.buttonCancel);

                var ims= director.getValueForKey("IMS")/2;
                this.buttonCancel.setPositionAnchor(.5,.5).setScale(ims,ims);
                this.buttonCancel.setLocation( director.width - this.buttonCancel.width*ims , director.height - this.buttonCancel.height*ims );

                return this;
            },

            show : function() {
                var me= this;

                if ( this.showing && this.layerShowId ) {
                    clearTimeout(this.layerShowId);
                }
                this.showing= true;
                this.showTime= new Date().getTime();
                this.parent.setZOrder(this, Number.MAX_VALUE);
                this.setVisible(true);
                this.enableEvents(true);
                this.buttonCancel.setVisible(false);

                this.buttonTTask= CAAT.getCurrentScene().setTimeout( 2500, function() {
                    me.buttonCancel.setVisible(true);
                });
            },

            hide : function() {
                var time= Math.max( 0, 666 - (new Date().getTime() - this.showTime ) );
                var me= this;

                this.layerShowId= setTimeout( function() {

                    me.cancel();
                }, time );
            },

            cancel : function() {
                var me= this;

                if ( me.buttonTTask ) {
                    me.buttonTTask.cancel();
                    me.buttonTTask= null;
                }

                me.setVisible(false);
                me.enableEvents(false);
                me.layerShowId= null;
            }

        },
        null,
        "iBasket.LoadingLayer"
    );

})();