CAAT.Module({
    defines : "iBasket.iBasketMenuScene",
    depends : [
        "CAAT.Module.LayoutUtils.RowLayout"
    ],


    extendsWith : function() {

        var W= 2048;
        var H= 1536;
        var WORLD_SCALE= 1;
        var SCENE_METERS= 5;
        var __director = null;
        var isMultiplayerChallenge= false;
        var multiplayerChallenge= null;

        function __startGame() {
            __director.getValueForKey("GameModel").reset();
            __director.easeInOut(
                1,
                CAAT.Scene.EASE_TRANSLATE,
                CAAT.Foundation.Actor.ANCHOR_RIGHT,
                0,
                CAAT.Foundation.Scene.EASE_TRANSLATE,
                CAAT.Foundation.Actor.ANCHOR_LEFT,
                1000,
                false,
                new CAAT.Interpolator().createExponentialInOutInterpolator(3,false),
                new CAAT.Interpolator().createExponentialInOutInterpolator(3,false) );
        }

        return {

            logo:       null,   // logo actor
                buttons:    null,   // buttons actor

            world:      null,   // box2d world
            scene:      null,   // CAAT scene

            dx:         0,
            dy:         -512,

            bodyPool:   null,
            rootActor:  null,
            ballsActor:  null,
            ludeiActor: null,
            backgroundActor: null,

            loadingLayer : null,

            soundActor : null,
            musicActor : null,

            button_leaderboards : null,
            button_achievements : null,

            initialize : function(director, font) {
                __director = director;
                this.font= font;
                this.createScene(director);
                this.createBackground(director);
//                this.createWorld(director);
                this.createLogo(director);
                this.createButtons(director);
                this.createLogin(director);
                this.createSoundControls(director);
                this.createLoadingLayer(director,font);
                this.createSocialSelection(director);
                if ( new Date().getMonth()===11 ) {
                    this.createSnowflakes(director);
                }
            },

            createSnowflakes : function(director) {
                var snowflakeImage= new CAAT.Foundation.SpriteImage("snowflake", 2, 3);

                function setup(actor) {
                    var type= Math.random();
                    if (type<.33) { type=1; } else if (type<.66) { type=2; } else { type=3; }

                    var x0= 50+(director.width-100)*Math.random();
                    var y0= -snowflakeImage.getHeight()-150*Math.random();
                    var x1= x0 + (Math.random() <.5?1:-1)*(50+50*Math.random());
                    var y1= director.height + 150*Math.random();

                    var r0= Math.random()*Math.PI*(Math.random() <.5?1:-1);

                    actor.setSpriteIndex((Math.random()*6)>>0);

                    var behavior= actor.behaviorList[0];
                    behavior.path.setLinear(x0,y0,x1,y1);
                    behavior.setDelayTime(0,(4000+Math.random()*3000)*type);
                    behavior.setTimeOffset(0);

                    var behavior1= actor.behaviorList[1];
                    behavior1.setValues( -r0, r0).setDelayTime(0,8000+4000*Math.random());

                    if (type===2) {
                        actor.setScale(.6,.6);
                    } else if (type===3) {
                        actor.setScale(.3,.3);
                    } else {
                        actor.setScale(1,1);
                    }
                }

                for( var i=0; i<60; i++ ) {

                    var snowflake= new CAAT.Foundation.Actor().
                        setBackgroundImage(snowflakeImage).
                        setSpriteIndex((Math.random()*6)>>0).
                        enableEvents(false).
                        addBehavior(
                            new CAAT.Behavior.PathBehavior().
                                setDelayTime(0,5000).
                                setValues( new CAAT.PathUtil.Path().setLinear( 0,0,0,0 )).
                                addListener( {
                                    behaviorExpired : function(behavior, scenetime, actor) {
                                        setup(actor);
                                    }
                                })
                        ).
                        addBehavior(
                            new CAAT.Behavior.RotateBehavior().
                                setCycle(true).
                                setInterpolator( new CAAT.Behavior.Interpolator().createExponentialInOutInterpolator(3,true) )
                        ).
                        rescaleToScreen();

                    setup(snowflake);
                    snowflake.behaviorList[0].setTimeOffset( Math.random() );
                    this.backgroundActor.addChild( snowflake );
                }
            },

            createLoadingLayer : function(director, font) {
                this.loadingLayer= iBasket.Util.createLoadingLayer(director, "spinner", font);
                this.scene.addChild( this.loadingLayer );
            },

            createSoundControls : function(director) {
                var c= iBasket.Util.createSoundControls(director,'menu_sound');
                c.setLocation( 20, 20 );
                c.rescaleToScreen();
                c.childrenList[1].scaleX*=1.5;
                c.childrenList[1].scaleY*=1.5;
                c.childrenList[0].scaleX*=1.5;
                c.childrenList[0].scaleY*=1.5;
                c.childrenList[1].x= 1.5*(c.childrenList[0].width)+10;

                this.backgroundActor.addChild(c);

                this.musicActor= c.getMusicActor();
                this.soundActor= c.getAudioActor();

                this.scene.addActorToInputList(this.musicActor,0);
                this.scene.addActorToInputList(this.soundActor,0);
            },


            createLogin : function(director) {

                if ( CocoonJS.SocialGaming.Facebook.IN ) {
                    return;
                }


                function inFB() {
                    if (!navigator.isCocoonJS) {
                        var url = (window.location != window.parent.location) ? document.referrer : document.location.hostname;
                        if (url.indexOf("facebook.ludei.com") !== -1) {
                            return true;
                        }
                    }
                    return false;
                };

                var me= this;

                var button= new CAAT.Foundation.Actor().
                    setAsButton(
                        new CAAT.Foundation.SpriteImage("menu_facebook_signup",2,2),
                        0,1,2,0,
                        function() {
                            me.loadingLayer.show();
                            CocoonJS.SocialGaming.Facebook.requestLogin();
                        });

                button.rescaleToScreen();

                var buttonlogout= new CAAT.Foundation.Actor().
                                    setAsButton(
                                        new CAAT.Foundation.SpriteImage("menu_facebook_signup",2,2),
                                        3,3,3,3,
                                        function() {
                                            me.loadingLayer.show();
                                            CocoonJS.SocialGaming.Facebook.requestLogout();
                                        });
                buttonlogout.rescaleToScreen();

                button.setPosition( director.width-button.width-10, 10 );
                buttonlogout.setPosition( director.width-button.width-10, 10).setVisible(false);

                this.backgroundActor.addChild(button);
                this.backgroundActor.addChild(buttonlogout);

                if (CocoonJS.SocialGaming.Facebook.isLoggedIn()) {
                    button.setVisible(false);
                    buttonlogout.setVisible(true);
                }

                CocoonJS.SocialGaming.Facebook.on("logout", function() {
                    button.setVisible(true);
                    buttonlogout.setVisible(false);
                    me.loadingLayer.hide();
                });


                CocoonJS.SocialGaming.Facebook.on("requestLoginFailed", function() {
                    if ( me.loadingLayer ) {
                        me.loadingLayer.hide();
                    }
                });

                CocoonJS.SocialGaming.Facebook.on("requestLoginSucceed", function(userInfo) {
                    CAAT.setCursor("default");
                    if ( me.loadingLayer ) {
                        me.loadingLayer.hide()
                    }
                    if ( inFB() ) {
                        button.setVisible(false);
                        buttonlogout.setVisible(false);
                    } else {
                        button.setVisible(false);
                        buttonlogout.setVisible(true);
                    }
                });



                this.scene.addActorToInputList(button,0);
                this.scene.addActorToInputList(buttonlogout,0);


            },

            createScene : function(director) {
                var me= this;

                this.scene = director.createScene().setId("iBasket.MenuScene");
                this.scene.enableInputList(2);

                var container= new CAAT.Foundation.ActorContainer().
                    setBackgroundImage( director.getImage('menu_background'), false).
                    setSize( director.width, director.height ).
                    setImageTransformation( CAAT.Foundation.SpriteImage.TR_FIXED_TO_SIZE );
                this.backgroundActor= container;
                this.scene.addChild(container) ;

                var scale= director.getValueForKey("WORLD_SCREEN_SCALE");

                this.rootActor= new CAAT.Foundation.ActorContainer().
                    setBounds(0,0,W,H).
                    setScaleAnchored( scale, scale, 0, 0 );
                this.scene.addChild( this.rootActor );
                WORLD_SCALE= scale;

                this.rootActor.setLocation( (director.width-W*scale)/2, 0 );

                this.scene.activated= function() {
                    iBasket.Util.tryShowRate();
                };

                this.scene.getIn= function() {
                    CocoonJS.Ad.hideBanner();
                    me.musicActor.setSpriteIndex( director.isMusicEnabled() ? 0 : 1 );
                    me.soundActor.setSpriteIndex( director.isSoundEffectsEnabled() ? 2 : 3 );

                };
            },

            createWorld : function(director) {

                CAAT.PMR= W/SCENE_METERS;

                var gravity =           new Box2D.Common.Math.b2Vec2(0, iBasket.Prefs.GRAVITY);
                var doSleep =           true;

                var BOUNDS_DENSITY=     1.2;
                var BOUNDS_FRICTION=    .9;
                var BOUNDS_RESTITUTION= .66;

                var dx0=                0; //this.dx;
                var dy0=                0; //this.dy;
                var dx1=                W;
                var dy1=                H;

                var i;

                var bounds= [
                    dx0,dy0,
                    dx1,dy0,

                    dx1,dy1,

                    dx0,dy1,

                    dx0,dy0
                ];

                this.world = new Box2D.Dynamics.b2World(gravity, doSleep);
//            this.world.SetContinuousPhysics(true);
                //this.world.SetSleepingAllowed(true);

                for( i=0; i<4; i++ )    {
                    var actor0= CAAT.Foundation.Box2D.B2DPolygonBody.createPolygonBody(
                        this.world,
                        {
                            bodyType:   Box2D.Dynamics.b2Body.b2_staticBody,
                            density:    BOUNDS_DENSITY,
                            friction:   BOUNDS_FRICTION,
                            restitution:BOUNDS_RESTITUTION,
                            polygonType:CAAT.Foundation.Box2D.B2DPolygonBody.TYPE.EDGE,
                            bodyDef:    [
                                {x: bounds[i*2  ],  y: bounds[i*2+1] },
                                {x: bounds[i*2+2],  y: bounds[i*2+3] }
                            ],
                            userData:   {}
                        });

                }

                // logo circular body
                this.circleBody( W/2, 500, 360 );
                // ludei logo circular body
                this.circleBodyFromActor( this.ludeiActor, Math.min(this.ludeiActor.width,this.ludeiActor.height)/2 );

                if (iBasket.Prefs.DEBUG ) {
                    CAAT.enableBox2DDebug(true,director,this.world, CAAT.PMR );
                }

                var me= this;
                this.scene.onRenderEnd =  function()    {
                    director.ctx.save();
                    director.ctx.translate( me.rootActor.x, me.rootActor.y );
                    director.ctx.scale( WORLD_SCALE, WORLD_SCALE );
                    me.world.DrawDebugData();
                    director.ctx.restore();
                }
            },

            createLogo : function(director) {

                var me= this;

                this.ballsActor= new CAAT.Foundation.ActorContainer().
                    setSize(W,H).
                    enableEvents(false);
                this.rootActor.addChild(this.ballsActor);

                var logoImg = director.getImage('menu_logo') ;
                var haloImg = director.getImage('menu_logo_halo');

                var logo= new CAAT.Foundation.Actor().
                    enableEvents(false).
                    setBackgroundImage( logoImg );
                logo.rescale();

                var logoHalo= new CAAT.Foundation.Actor().
                    setBackgroundImage( haloImg ).
                    setScale(0.01,0.01).
                    enableEvents(false).
                    addBehavior( new CAAT.Behavior.RotateBehavior().
                        setValues(0,2*Math.PI).
                        setCycle(true).
                        setDelayTime(0,60000)
                ).
                    addBehavior( new CAAT.Behavior.ScaleBehavior().
                        setValues( 0.01,1.15, 0.01,1.15).
                        setDelayTime( 1500, 10000).
                        setInterpolator( new CAAT.Behavior.Interpolator().createElasticOutInterpolator(1.1,.3))
                    );

                logoHalo.rescale();

                var l= new CAAT.Foundation.ActorContainer().
                    enableEvents(false).
                    setSize(logoHalo.width, logoHalo.height).
                    setPositionAnchor(.5,.5);
                l.setLocation( 0, -l.height );

                l.addBehavior(
                    new CAAT.Behavior.PathBehavior().
                        setValues(
                            new CAAT.PathUtil.Path().
                                setLinear(
                                    W/2, -l.height,
                                    W/2,  l.height/2
                                )
                        ).
                        setDelayTime( 2000, 1500).
                        setInterpolator(
                        new CAAT.Behavior.Interpolator().createBounceOutInterpolator()
                    )
                );

                logo.centerAt(l.width/2, l.height/2);
                logoHalo.centerAt(l.width/2, l.height/2);

                this.rootActor.addChild(l);
                l.addChild(logoHalo);
                l.addChild(logo);
            },

            createButtons : function(director) {

                var createButton= function(img, insideImg, callback, label) {

                    var c= new CAAT.Foundation.ActorContainer();

                    var b = new CAAT.Foundation.ActorContainer().
                        setAsButton( img, 0, 1, 2, 3, callback );
                    b.rescale();

                    c.setSize(b.width, b.height);

                    if ( insideImg ) {
                        var bb= new CAAT.Foundation.Actor().
                            setBackgroundImage(insideImg).
                            enableEvents(false);
                        bb.rescale().
                            centerAt(b.width/2, b.height/2 - 2);
                        b.addChild(bb);
                    }

                    if ( label) {
                        var l= new CAAT.Foundation.Actor().setBackgroundImage(label).rescale();
                        l.setLocation((b.width- l.width)/2, b.height);
                        c.addChild( l );
                        c.height+= l.height;
                    }

                    c.addChild(b);
                    c.__width= c.width;
                    c.__height= c.height;

                    return c;
                };

                var buttons_container= new CAAT.Foundation.ActorContainer();
                var button_image= new CAAT.Foundation.SpriteImage().initialize(director.getImage("menu_button_round"),1,4);

                var buttons= [
                    createButton( button_image, director.getImage("menu_icon_scores"), this.__showLeaderBoards.bind(this), "txt_scores" ),
                    createButton( button_image, director.getImage("menu_icon_achievements"), this.__showAchievements.bind(this), "txt_achievements" ),
                    createButton( button_image, director.getImage("menu_icon_shop"), this.__buyAds.bind(this), "txt_shop" ),
                    createButton( button_image, director.getImage("menu_icon_restore"), this.__restoreAds.bind(this), "txt_restore" ),
                    createButton( button_image, director.getImage("menu_icon_moregames"), this.__showMoreGames.bind(this), "txt_moregames" )
                ];

                for( var i=0; i<buttons.length; i++ ) {
                    buttons_container.addChild( buttons[i].
                        setScale(0.01,0.01).
                        addBehavior(
                            new CAAT.Behavior.ScaleBehavior().
                                setValues(0.01,1, 0.01,1).
                                setDelayTime( 1000 + i*100, 2000).
                                setInterpolator(
                                    new CAAT.Behavior.Interpolator().createElasticOutInterpolator(1,.2)
                                )
                        )
                    );
                }

                var menu= new CAAT.Foundation.ActorContainer();

                var button_play =
                    iBasket.Util.createButtonWithTextAndIcon(
                            new CAAT.Foundation.SpriteImage().initialize(director.getImage("menu_button_play"), 2, 2),
                            "PLAY",
                            director.getImage("menu_icon_play"),
                            this.font,
                            __startGame
                        ).
                        setScale(.01, .01).
                        addBehavior(
                            new CAAT.Behavior.ScaleBehavior().
                                setValues(0.01, 1, 0.01, 1).
                                setDelayTime(1000, 2000).
                                setInterpolator(
                                    new CAAT.Behavior.Interpolator().createElasticOutInterpolator(1, .2)
                                )
                        ).
                        setId("button_play");

                var buttons_play= new CAAT.Foundation.ActorContainer().
                    setLayout(
                        new CAAT.Foundation.UI.Layout.BoxLayout().
                            setAxis( CAAT.Foundation.UI.Layout.LayoutManager.AXIS.X).
                            setVerticalAlignment( CAAT.Foundation.UI.Layout.LayoutManager.ALIGNMENT.BOTTOM).
                            setHGap(20)
                    );

                buttons_play.addChild(button_play);

                this.scene.addActorToInputList(button_play,0);

                menu.addChild( buttons_play );
                menu.addChild(buttons_container.setScale(1.3, 1.3));

                this.menuActor= menu;

                this.relayoutMenu();

                this.rootActor.addChild( menu );
            },

            relayoutMenu : function() {


                var menu= this.menuActor;
                var button_play= menu.childrenList[0];
                var buttons_container= menu.childrenList[1];
                var buttonslength= buttons_container.childrenList.length;

                if ( !navigator.isCocoonJS || !iBasket.Prefs.ADS_ENABLED ) {
                    buttons_container.childrenList[2].visible= false;
                    buttons_container.childrenList[3].visible= false;
                    buttonslength= buttons_container.childrenList.length - 2;
                } else {
                    buttons_container.childrenList[2].visible= true;
                    buttons_container.childrenList[3].visible= true;
                }

                buttons_container.setSize( buttons_container.childrenList[0].__width*buttonslength + (buttonslength-1)*100, buttons_container.childrenList[0].__height );
                var layout_buttons_container= new CAAT.Foundation.UI.Layout.GridLayout(0,buttonslength).setPadding(50,50,0,0).setHGap(0);

                layout_buttons_container.doLayout(buttons_container);

                var layout_menu= new CAAT.Foundation.UI.Layout.BoxLayout().
                    setAxis(CAAT.Foundation.UI.Layout.LayoutManager.AXIS.Y).
                    setHorizontalAlignment(CAAT.Foundation.UI.Layout.LayoutManager.ALIGNMENT.CENTER).
                    setVerticalAlignment(CAAT.Foundation.UI.Layout.LayoutManager.ALIGNMENT.CENTER).
                    setVGap(40);


                var buttonsScale= 1.1;
                menu.setSize(
                    (button_play.width + buttons_container.width) * buttonsScale,
                    (button_play.height + buttons_container.height)*2 * buttonsScale );

                layout_menu.doLayout( menu );

                menu.setLocation( (W - menu.width)/2, H/2 + 280 );
                menu.setScale(buttonsScale,buttonsScale);
            },

            __buyAds : function() {

                var me= this;

                var products= CAAT.currentDirector.getValueForKey("iBasket.Products");
                if (!products) {
                    return;
                }

                iBasket.Util.buyOrRestoreAds(
                    function onPurchaseStarted(productId) {
                        me.loadingLayer.show();
                    },
                    function onPurchaseFailed(productId, error) {
                        me.loadingLayer.hide();
                        alert("Purchase of Item '" + products[0].title + "' failed: " + error);
                    },
                    function onPurchaseCompleted(purchaseInfo) {
                        me.loadingLayer.hide();
                        me.relayoutMenu();
                    });

                CocoonJS.Store.purchaseProduct(products[0].productId);

            },

            __restoreAds : function() {
                var me= this;


                iBasket.Util.buyOrRestoreAds(
                    function onPurchaseStarted(productId) {
                        me.loadingLayer.show();
                    },
                    function onPurchaseFailed(productId, error) {
                        me.loadingLayer.hide();
                        CocoonJS.App.showMessageBox(
                            "Restore Purchase",
                            "Purchase restoration failed with error: ':"+error+"'.",
                            "Ok");
                    },
                    function onPurchaseCompleted(purchaseInfo) {
                        me.loadingLayer.hide();
                        me.relayoutMenu();
                    });


                CocoonJS.Store.restorePurchases();
            },

            __showMoreGames : function() {

                var url= iBasket.Prefs.COCOONJS_LOCAL_MOREGAMES_PATH_URI;
                var me= this;

                if ( navigator.isCocoonJS ) {
                    this.loadingLayer.show();

                    CocoonJS.App.onLoadInTheWebViewSucceed.addEventListenerOnce(function () {
                        me.loadingLayer.hide();
                        CocoonJS.App.showTheWebView();
                    });

                    CocoonJS.App.onLoadInTheWebViewFailed.addEventListenerOnce( function() {
                        me.loadingLayer.hide();
                        alert("Cant´t show more games information.");
                    });

                    if (/ios/.test(navigator.userAgent)) {
                        url+= "ios.html";
                    }
                    else if (/android/.test(navigator.userAgent)) {
                        url+= "android.html";
                    }

                    CocoonJS.App.loadInTheWebView(url);
                } else {
                    url= iBasket.Prefs.COCOONJS_REMOTE_MOREGAMES_PATH_URI;
                    CocoonJS.App.openURL(url);
                }


            },

            createSocialSelection : function(director) {
                this.socialSelection= iBasket.Util.createSocialSelection(director);
                this.scene.addChild(this.socialSelection);
            },

            __showLeaderBoards : function() {

                var fn= function(srv, me) {
                    me.loadingLayer.show();
                    srv.onLeaderboardViewFailed.addEventListenerOnce(function() {
                        me.loadingLayer.hide();
                        alert("Error loading Facebook leaderboards.");
                    });
                    srv.onLeaderboardViewSucceed.addEventListenerOnce(function() {
                        me.loadingLayer.hide();
                    });
                    srv.showLeaderboardView();
                    me.socialSelection.setVisible(false);
                }

                iBasket.Util.setupSocialSelection(this.socialSelection, fn, this);
            },

            __showAchievements : function() {

                var fn= function(srv, me) {
                    me.loadingLayer.show();
                    srv.onAchievementsViewFailed.addEventListenerOnce(function() {
                        me.loadingLayer.hide();
                        alert("Error loading Facebook achievements.");
                    });
                    srv.onAchievementsViewSucceed.addEventListenerOnce(function() {
                        me.loadingLayer.hide();
                    });
                    srv.showUserAchievementsView();
                    me.socialSelection.setVisible(false);
                }

                iBasket.Util.setupSocialSelection(this.socialSelection, fn, this);

            },
/*
            circleBodyFromActor : function(actor,actorRadius) {
                this.circleBody( actor.x + actor.width/2, actor.y + actor.height/2, actorRadius );
            },

            circleBody : function(x,y,r) {
                CAAT.Foundation.Box2D.B2DCircularBody.createCircularBody(
                    this.world,
                    {
                        x:                      x,
                        y:                      y,
                        bodyType:               Box2D.Dynamics.b2Body.b2_staticBody,
                        density:                1,
                        restitution:            1,
                        friction:               1,
                        image:                  null,
                        radius:                 r,
                        bodyDefScale:           1,
                        bodyDefScaleTolerance:  0,
                        userData:               {}
                    }
                )
            },

            bodyFromActor : function(actor) {
                CAAT.Foundation.Box2D.B2DPolygonBody.createPolygonBody(
                    this.world,
                    {
                        x:                      actor.x- actor.width*actor.tAnchorX,
                        y:                      actor.y- actor.height*actor.tAnchorY,
                        bodyType:               Box2D.Dynamics.b2Body.b2_staticBody,
                        density:                1,
                        restitution:            1,
                        friction:               1,
                        image:                  null,
                        polygonType:            CAAT.Foundation.Box2D.B2DPolygonBody.TYPE.BOX,
                        bodyDef:                [
                            {
                                x: actor.x - actor.width*actor.tAnchorX,
                                y: actor.y - actor.height*actor.tAnchorY
                            },
                            {
                                x: actor.x - actor.width*actor.tAnchorX + actor.width,
                                y: actor.y - actor.height*actor.tAnchorY + actor.height
                            }
                        ],
                        bodyDefScale:           1,
                        bodyDefScaleTolerance:  0,
                        userData:               {}
                    }
                )
            },
*/
            /**
             * Create a background bouncing ball every 1 second.
             *
             * @param director {CAAT.Director}
             */
            createBackground : function(director) {
                var me= this;
                var i,j;

                // create some clouds that traverse the screen forever
                for( var i=0; i<4; i++ ) {
                    var h= Math.random()*director.height;
                    var im= director.getImage("menu_cloud"+(i%3) );

                    var cloud=
                        new CAAT.Foundation.Actor().
                            setBackgroundImage( im ).
                            enableEvents(false).
                            addBehavior(
                                new CAAT.Behavior.PathBehavior().
                                    setValues(
                                        new CAAT.PathUtil.Path().
                                            setLinear(
                                                -im.width-director.width/5*Math.random(),
                                                h,
                                                director.width+director.width/5*Math.random(),
                                                h
                                            )
                                    ).
                                    setDelayTime( 0, 40000+40000*Math.random()).
                                    addListener( {
                                        behaviorExpired : function( behavior, time, actor) {
                                            h= Math.random()*director.height;
                                            behavior.path.setLinear(
                                                -actor.width-Math.random()*actor.width, h,
                                                director.width+actor.width, h );
                                            behavior.setTimeOffset(0).setDelayTime( 0, 40000+40000*Math.random());
                                        }
                                    }).
                                    setTimeOffset( Math.random() )
                            );
                    cloud.rescaleToScreen();

                    me.backgroundActor.addChild(cloud);

                }

                // create ludei logo
                this.ludeiActor= new CAAT.Foundation.Actor().
                                setAsButton(new CAAT.Foundation.SpriteImage(director.getImage("menu_ludei"),1,1),
                                 0, 0, 0, 0, 
                                 function() { CocoonJS.App.openURL("http://www.ludei.com")});
                                
                this.ludeiActor.rescaleToScreen();
                this.ludeiActor.setLocation( director.width-this.ludeiActor.width-10, director.height-this.ludeiActor.height-10 );
                this.backgroundActor.addChild( this.ludeiActor );
                this.scene.addActorToInputList(this.ludeiActor,0);

/*
                // create a basketball each 500 ms.
                this.scene.createTimer(
                    3500,
                    500,
                    function( scene_time, task_elapsed_time, timertask_instance) {

                        var x0= Math.random()*W;
                        var y0= Math.random()*H;
                        var x1= Math.random()*W;
                        var y1= Math.random()*H;
                        var index= (Math.random()*iBasket.GameModel.MAX_BALLS_TYPE)>>0;

                        var ball= iBasket.GameModel.prototype.createBall(
                            director, me.world, x0, y0, (Math.random()*6)>>0 ).
                            setDiscardable(true).
                            enableEvents(false);

                        var velocity= iBasket.BallContainer.prototype.calculateVelocity(
                            x0,y0,x1,y1 );

                        ball.setFrameTime( scene_time, 4000 );

                        ball.worldBody.SetLinearVelocity(
                            new Box2D.Common.Math.b2Vec2(
                                velocity.x,
                                velocity.y) );
                        ball.worldBody.SetAngularVelocity( Math.PI*Math.random() );

                        me.ballsActor.addChild(ball);
                        timertask_instance.reset( scene_time );
                    },
                    null,
                    null);
                    */
            }
        }
    }


});
