CAAT.Module({
    defines:"iBasket.Util",

    constants:{

        VERSION : "iBasket.Rate.Cloud1.4",

        createButtonWithTextAndIcon:function (img, text, icon, font, callback) {

            var ims= CAAT.currentDirector.getValueForKey("IMS");
            var b = new CAAT.Foundation.ActorContainer().
                setAsButton(img, 0, 1, 2, 0, callback);
            b.setSize(b.width/ims, b.height/ims).
                setImageTransformation(CAAT.Foundation.SpriteImage.TR_FIXED_TO_SIZE);

            var actors = [];

            if (icon) {
                var aicon= new CAAT.Foundation.Actor().
                    setBackgroundImage(icon).
                    enableEvents(false);
                aicon.setSize(aicon.width/ims, aicon.height/ims).
                    setImageTransformation( CAAT.Foundation.SpriteImage.TR_FIXED_TO_SIZE );
                actors.push(aicon);
            }
            if (text) {
                actors.push(new CAAT.Foundation.UI.TextActor().
                    setFont(font).
                    setText(text)
                );
            }

            if (actors.length) {
                var c = new CAAT.Foundation.ActorContainer().
                    setSize(b.width, b.height).
                    enableEvents(false).
                    setLayout(
                        new CAAT.Foundation.UI.Layout.BoxLayout().
                            setAxis(CAAT.Foundation.UI.Layout.LayoutManager.AXIS.X).
                            setHGap(20).
                            setAllPadding(5).
                            setVerticalAlignment(CAAT.Foundation.UI.Layout.LayoutManager.ALIGNMENT.CENTER).
                            setHorizontalAlignment(CAAT.Foundation.UI.Layout.LayoutManager.ALIGNMENT.CENTER)
                    );
                for (var i = 0; i < actors.length; i++) {
                    c.addChild(actors[i]);
                }

                c.centerAt(b.width / 2, b.height / 2);
                b.addChild(c);
            }

            return b;
        },

        createSoundControls : function(director, soundImage) {
            var c= new CAAT.Foundation.ActorContainer( CAAT.Foundation.ActorContainer.AddHint.CONFORM );

            var im= new CAAT.Foundation.SpriteImage( director.getImage(soundImage),1,4);
            var music= new CAAT.Foundation.Actor().setBackgroundImage(im);
            var audio= new CAAT.Foundation.Actor().setBackgroundImage(im);

            music.mouseUp= function(e) {
                director.setMusicEnabled( !director.isMusicEnabled() );
                CAAT.Module.Storage.LocalStorage.save("iBasket.Music", director.isMusicEnabled());
                e.source.setSpriteIndex( director.isMusicEnabled() ? 0 : 1 );
                if ( director.isMusicEnabled() ) {
                    director.musicPlay("music");
                } else {
                    director.musicStop();
                }
            };
            audio.mouseUp= function(e) {
                director.setSoundEffectsEnabled( !director.isSoundEffectsEnabled() );
                CAAT.Module.Storage.LocalStorage.save("iBasket.Sound", director.isSoundEffectsEnabled());
                e.source.setSpriteIndex( director.isSoundEffectsEnabled() ? 2 : 3 );
            };

            c.addChild(music);
            audio.x= music.width + 20;
            c.addChild(audio);

            c.getAudioActor= function() {
                return audio;
            };

            c.getMusicActor= function() {
                return music;
            };

            return c;
        },

        createLoadingLayer : function(director, spinnerImg, font) {
            return new iBasket.LoadingLayer(director, spinnerImg, font);
        },

        createSocialSelection : function(director) {
            var socialSelection = new CAAT.Foundation.ActorContainer().
                setFillStyle('rgba(0,0,0,.7)').
                setSize(director.width, director.height).
                setVisible(false);
            socialSelection.mouseUp = function (e) {
                e.source.setVisible(false);
            }
            socialSelection.show = function () {
                this.setVisible(true).
                    emptyBehaviorList().
                    addBehavior(
                    new CAAT.Behavior.AlphaBehavior().
                        setValues(0, 1).
                        setDelayTime(0, 500)
                );
            };

            return socialSelection;
        },

        setupSocialSelection : function(socialSelection, fn, context) {
            var srvs= CocoonJS.SocialGaming.SocialGamingManager.getRegisteredSocialServicesWithView();

            if (srvs.length===0 ) {
                alert("Leaderboards and Achievements are only available if logged in to Facebook or Game Center.");
            } else
            if (srvs.length===1 ) {
                fn(srvs[0], context);
            } else {
                socialSelection.emptyChildren();
                for( var i=0; i<srvs.length; i+=1) {
                    var socialGamingUserData= CocoonJS.SocialGaming.SocialGamingManager.getSocialServiceUserData(srvs[i]);

                    var socialActor= null;
                    if (null===socialGamingUserData) {
                        console.log("SocialService "+srvs[i].extensionName+" has no userData.");
                        socialActor= new CAAT.Foundation.UI.TextActor().
                            setFont( "30px Arial" ).
                            setText( srvs[i].extensionName );
                    } else {
                        socialActor= new CAAT.Foundation.Actor().
                            setAsButton(
                                new CAAT.Foundation.SpriteImage(socialGamingUserData.image,1,6),
                                socialGamingUserData.frames[0],
                                socialGamingUserData.frames[1],
                                socialGamingUserData.frames[2],
                                socialGamingUserData.frames[0],
                                fn.bind(null, srvs[i], context)
                            );
                    }

                    socialActor.rescaleToScreen();
                    socialSelection.addChild( socialActor );

                    var layout= new CAAT.Foundation.UI.Layout.BoxLayout().
                        setAxis( CAAT.Foundation.UI.Layout.LayoutManager.AXIS.X).
                        setHGap(40).
                        setHorizontalAlignment( CAAT.Foundation.UI.Layout.LayoutManager.ALIGNMENT.CENTER).
                        setVerticalAlignment( CAAT.Foundation.UI.Layout.LayoutManager.ALIGNMENT.CENTER);

                    layout.doLayout(socialSelection);
                }

                socialSelection.show();
            }
        },

        rateTheApp : function() {
            CAAT.Module.Storage.LocalStorage.save( iBasket.Util.VERSION ,1);

            var IOS_RATING_URL = iBasket.Prefs.RATING_URL_IOS;
            var ANDROID_RATING_URL = iBasket.Prefs.RATING_URL_ANDROID;

            // Select the rating URL depending on the operating system and if we are inside CocoonJS.
            var ratingURL = "http://ludei.com";
            if (navigator.isCocoonJS) {
                if (/ios/.test(navigator.userAgent)) {
                    ratingURL = IOS_RATING_URL;
                }
                else if (/android/.test(navigator.userAgent)) {
                    ratingURL = ANDROID_RATING_URL;
                }
            } else {
                return;
            }

            CocoonJS.App.openURL(ratingURL);

            var gameModel= CAAT.currentDirector.getValueForKey("GameModel");
            gameModel.setApplicationRated();
        },

        tryShowRate : function() {

            var ratePolicy= CAAT.currentDirector.getValueForKey("RatingPolicy");

            if ( ratePolicy.isPolicyMet() ) {
                CocoonJS.App.onMessageBoxConfirmed.addEventListenerOnce( function() {
                    CAAT.currentDirector.setValueForKey("iBasket.inRate",false);
                    CAAT.currentDirector.getValueForKey("iBasket.ResultScene").rateThisApp();
                    ratePolicy.reset();
                });
                CocoonJS.App.onMessageBoxDenied.addEventListenerOnce( function() {
                    CAAT.currentDirector.setValueForKey("iBasket.inRate",false);
                    ratePolicy.reset();
                });
                if ( !CAAT.Module.Storage.LocalStorage.load(iBasket.Util.VERSION) ) {
                    CocoonJS.App.showMessageBox(
                        "Rate iBasket",
                        "If you enjoy using iBasket, would you mind taking a moment to rate this update? Thanks!",
                        "Rate iBasket",
                        "Remind me later");
                }
                CAAT.currentDirector.setValueForKey("iBasket.inRate",true);
            }
        },

        buyOrRestoreAds : function( onPurchaseStarted, onPurchaseFailed, onPurchaseSucceed ) {

            function fn0(productId) {
                onPurchaseStarted(productId);
            }

            function fn1(productId, error) {
                onPurchaseFailed(productId, error);
            }

            function fn2(purchaseInfo) {

                var msg= "Unkown Operation";
                switch (purchaseInfo.purchaseState) {
                    case CocoonJS.Store.PurchaseState.PURCHASED:
                        CAAT.currentDirector.getValueForKey("iBasket.REMOVE_ADS")();
                        msg= "Product '" + purchaseInfo.productId + "' successfully purchased!";
                        break;

                    case CocoonJS.Store.PurchaseState.CANCELED:
                        msg= "Product '" + purchaseInfo.productId + "' purchase has been canceled";
                        break;

                    case CocoonJS.Store.PurchaseState.REFUNDED:
                        msg= "Product '" + purchaseInfo.productId + "' purchase has been refunded";
                        break;

                    case CocoonJS.Store.PurchaseState.EXPIRED:
                        msg= "Product '" + purchaseInfo.productId + "' purchase has expired";
                        break;
                }

                console.log(msg);

                CocoonJS.Store.addPurchase(purchaseInfo);
                CocoonJS.Store.consumePurchase(purchaseInfo.transactionId, purchaseInfo.productId);
                CocoonJS.Store.finishPurchase(purchaseInfo.transactionId);

                onPurchaseSucceed(msg);

            }

            CocoonJS.Store.onProductPurchaseStarted.addEventListenerOnce(fn0);
            CocoonJS.Store.onProductPurchaseFailed.addEventListenerOnce(fn1);
            CocoonJS.Store.onProductPurchaseCompleted.addEventListenerOnce( fn2);

        }
    }
});
